package org.apache.bcel.classfile;

import java.util.Arrays;
import java.io.DataOutputStream;
import org.apache.commons.lang3.ArrayUtils;
import java.io.IOException;
import java.io.DataInput;

public final class ExceptionTable extends Attribute
{
    private int[] exceptionIndexTable;
    
    public ExceptionTable(final ExceptionTable c) {
        this(c.getNameIndex(), c.getLength(), c.getExceptionIndexTable(), c.getConstantPool());
    }
    
    ExceptionTable(final int nameIndex, final int length, final DataInput input, final ConstantPool constantPool) throws IOException {
        this(nameIndex, length, (int[])null, constantPool);
        final int exceptionCount = input.readUnsignedShort();
        this.exceptionIndexTable = new int[exceptionCount];
        for (int i = 0; i < exceptionCount; ++i) {
            this.exceptionIndexTable[i] = input.readUnsignedShort();
        }
    }
    
    public ExceptionTable(final int nameIndex, final int length, final int[] exceptionIndexTable, final ConstantPool constantPool) {
        super((byte)3, nameIndex, length, constantPool);
        this.exceptionIndexTable = ((exceptionIndexTable != null) ? exceptionIndexTable : ArrayUtils.EMPTY_INT_ARRAY);
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitExceptionTable(this);
    }
    
    @Override
    public Attribute copy(final ConstantPool constantPool) {
        final ExceptionTable c = (ExceptionTable)this.clone();
        if (this.exceptionIndexTable != null) {
            c.exceptionIndexTable = this.exceptionIndexTable.clone();
        }
        c.setConstantPool(constantPool);
        return c;
    }
    
    @Override
    public void dump(final DataOutputStream file) throws IOException {
        super.dump(file);
        file.writeShort(this.exceptionIndexTable.length);
        for (final int index : this.exceptionIndexTable) {
            file.writeShort(index);
        }
    }
    
    public int[] getExceptionIndexTable() {
        return this.exceptionIndexTable;
    }
    
    public String[] getExceptionNames() {
        final String[] names = new String[this.exceptionIndexTable.length];
        Arrays.setAll(names, i -> Utility.pathToPackage(super.getConstantPool().getConstantString(this.exceptionIndexTable[i], (byte)7)));
        return names;
    }
    
    public int getNumberOfExceptions() {
        return (this.exceptionIndexTable == null) ? 0 : this.exceptionIndexTable.length;
    }
    
    public void setExceptionIndexTable(final int[] exceptionIndexTable) {
        this.exceptionIndexTable = ((exceptionIndexTable != null) ? exceptionIndexTable : ArrayUtils.EMPTY_INT_ARRAY);
    }
    
    @Override
    public String toString() {
        final StringBuilder buf = new StringBuilder();
        buf.append("Exceptions: ");
        for (int i = 0; i < this.exceptionIndexTable.length; ++i) {
            final String str = super.getConstantPool().getConstantString(this.exceptionIndexTable[i], (byte)7);
            buf.append(Utility.compactClassName(str, false));
            if (i < this.exceptionIndexTable.length - 1) {
                buf.append(", ");
            }
        }
        return buf.toString();
    }
}
